@@ -1,9 +1,9 @@
 /*
- *  $Id: util.c,v 1.302 2021/03/06 17:32:24 tom Exp $
+ *  $Id: util.c,v 1.310 2024/03/06 22:43:58 tom Exp $
  *
  *  util.c -- miscellaneous utilities for dialog
  *
- *  Copyright 2000-2020,2021	Thomas E. Dickey
+ *  Copyright 2000-2023,2024	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -24,23 +24,8 @@
  *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include <dialog.h>
-#include <dlg_keys.h>
 #include <dlg_internals.h>
-
-#include <sys/time.h>
-
-#ifdef HAVE_SETLOCALE
-#include <locale.h>
-#endif
-
-#ifdef NEED_WCHAR_H
-#include <wchar.h>
-#endif
-
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
+#include <dlg_keys.h>
 
 #if defined(NCURSES_VERSION)
 #define CAN_KEEP_TITE 1
@@ -769,6 +754,34 @@ dlg_print_listitem(WINDOW *win,
     }
 }
 
+void
+dlg_print_nowrap(WINDOW *win, const char *line, int width)
+{
+    int limit;
+    int x_1st;
+    int x_now;
+    int x = 0;
+    int y;
+    int ch;
+    bool done;
+
+    (void) y;
+    getyx(win, y, x_1st);
+    waddch(win, ' ');
+    done = (width <= MARGIN);
+    limit = x_1st + width - 1;
+    do {
+	if (done) {
+	    waddch(win, ' ');
+	} else if ((ch = line[x++]) != '\0') {
+	    waddch(win, UCH(ch));
+	} else {
+	    done = TRUE;
+	}
+	getyx(win, y, x_now);
+    } while (x_now < limit);
+}
+
 /*
  * Print up to 'cols' columns from 'text', optionally rendering our escape
  * sequence for attributes and color.
@@ -802,7 +815,7 @@ dlg_print_text(WINDOW *win, const char *txt, int cols, chtype *attr)
 		int code;
 
 		txt += 2;
-		switch (code = CharOf(*txt)) {
+		switch (code = UCH(*txt)) {
 #ifdef HAVE_COLOR
 		case '0':
 		case '1':
@@ -870,7 +883,7 @@ dlg_print_text(WINDOW *win, const char *txt, int cols, chtype *attr)
 	 * tabs are nonprinting, and curses will expand tabs to one or
 	 * more blanks.
 	 */
-	thisTab = (CharOf(*txt) == TAB);
+	thisTab = (UCH(*txt) == TAB);
 	if (dialog_state.text_only) {
 	    x_before = x_after;
 	} else {
@@ -880,7 +893,7 @@ dlg_print_text(WINDOW *win, const char *txt, int cols, chtype *attr)
 	    }
 	}
 	if (dialog_state.text_only) {
-	    int ch = CharOf(*txt++);
+	    int ch = UCH(*txt++);
 	    if (thisTab) {
 		while ((x_after++) % 8) {
 		    fputc(' ', dialog_state.output);
@@ -890,7 +903,7 @@ dlg_print_text(WINDOW *win, const char *txt, int cols, chtype *attr)
 		x_after++;	/* FIXME: handle meta per locale */
 	    }
 	} else {
-	    (void) waddch(win, CharOf(*txt++) | useattr);
+	    (void) waddch(win, UCH(*txt++) | useattr);
 	    getyx(win, y_after, x_after);
 	}
 	if (thisTab && (y_after == y_origin))
@@ -1872,8 +1885,8 @@ dlg_exit(int code)
 
 #ifdef NO_LEAKS
     _dlg_inputstr_leaks();
-#if defined(NCURSES_VERSION) && (defined(HAVE_CURSES_EXIT) || defined(HAVE__NC_FREE_AND_EXIT))
-    curses_exit(code);
+#if defined(NCURSES_VERSION) && (defined(HAVE_EXIT_CURSES) || defined(HAVE__NC_FREE_AND_EXIT))
+    exit_curses(code);
 #endif
 #endif
 
@@ -1955,11 +1968,15 @@ dlg_exiterr(const char *fmt, ...)
 
     end_dialog();
 
-    (void) fputc('\n', stderr);
-    va_start(ap, fmt);
-    (void) vfprintf(stderr, fmt, ap);
-    va_end(ap);
-    (void) fputc('\n', stderr);
+    if (fmt != NULL) {
+	(void) fputc('\n', stderr);
+	va_start(ap, fmt);
+	(void) vfprintf(stderr, fmt, ap);
+	va_end(ap);
+	(void) fputc('\n', stderr);
+    } else {
+	fputs("dialog?\n", stderr);
+    }
 
 #ifdef HAVE_DLG_TRACE
     va_start(ap, fmt);
@@ -2697,13 +2714,19 @@ dlg_set_focus(WINDOW *parent, WINDOW *win)
 }
 
 /*
- * Returns the nominal maximum buffer size.
+ * Returns the nominal maximum buffer size, given the caller's estimate of
+ * the needed size.  If the parameter is not greater than zero, return the
+ * configured buffer size.
  */
 int
 dlg_max_input(int max_len)
 {
-    if (dialog_vars.max_input != 0 && dialog_vars.max_input < MAX_LEN)
-	max_len = dialog_vars.max_input;
+    int limit = ((dialog_vars.max_input > 0)
+		 ? dialog_vars.max_input
+		 : MAX_LEN);
+
+    if (max_len > limit || max_len <= 0)
+	max_len = limit;
 
     return max_len;
 }
